<?php

declare(strict_types=1);

namespace Drupal\sites_migrator;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Component\Console\Output\ConsoleOutput;

final class DomainModuleService {

  /**
   * @param \Drupal\sites_migrator\ContentInheritService $parent
   *   The parent content entity service.
   */
  public function __construct(
    readonly private ContentInheritService $parent,
    readonly private ModuleHandlerInterface $moduleHandler,
    readonly private EntityTypeManagerInterface $entityTypeManager,
  ) {
  }

  public function inheritDomainContent(?ConsoleOutput $output = NULL): void {
    if (!$this->moduleHandler->moduleExists('domain')) {
      throw new \Exception('The "domain" module is not installed. Please install and enable it to use this functionality.');
    }

    $nodeQuery = $this->parent->entityTypeManager
      ->getStorage('node')
      ->getQuery()
      ->condition('field_domain_access', NULL, 'IS NOT NULL')
      ->accessCheck(FALSE);
    $nids = $nodeQuery->execute();

    $progressBar = NULL;
    if ($output) {
      $progressBar = new ProgressBar($output, count($nids));
      $progressBar->start();
    }

    foreach ($nids as $nid) {
      $node = $this->entityTypeManager
        ->getStorage('node')->load($nid);

      $domainId = $node->get('field_domain_access')->getString();

      $domainQuery = $this->parent->entityTypeManager
        ->getStorage('domain')
        ->getQuery()
        ->condition('id', $domainId)
        ->accessCheck(FALSE);
      $domainId = $domainQuery->execute();
      // @phpstan-ignore class.notFound
      $domain = $this->entityTypeManager
        ->getStorage('node')->load(current($domainId));

      $groupQuery = $this->parent->entityTypeManager
        ->getStorage('group')
        ->getQuery()
        ->condition('field_hostname', $domain->getHostname())
        ->accessCheck(FALSE);
      $gidResult = $groupQuery->execute();

      if (count($gidResult) === 1) {
        $gid = current($gidResult);
        $this->parent->ensureSiteContextByGroup($node, $gid);
      }

      if ($progressBar) {
        $progressBar->advance();
      }
    }

    if ($progressBar) {
      $progressBar->finish();
      $output->writeln('');
    }
  }

}
